@@ -42,7 +42,7 @@ from urlparse import urlparse
 
 import dom0_pluginlib
 import utils
-
+import XenAPI
 
 dom0_pluginlib.configure_logging('glance')
 logging = dom0_pluginlib.logging
@@ -297,7 +297,7 @@ def _upload_tarball_by_url_v1(staging_path, image_id, glance_endpoint,
         conn.close()
 
 
-def _update_image_meta_v2(conn, image_id, extra_headers, properties):
+def _update_image_meta_v2(conn, extra_headers, properties, patch_path):
     # NOTE(sirp): There is some confusion around OVF. Here's a summary
     # of where we currently stand:
     #   1. OVF as a container format is misnamed. We really should be
@@ -321,8 +321,8 @@ def _update_image_meta_v2(conn, image_id, extra_headers, properties):
                 "op": "add"}
         body.append(prop)
     body = json.dumps(body)
-    conn.request('PATCH', '/v2/images/%s' % image_id,
-                 body=body, headers=headers)
+
+    conn.request('PATCH', patch_path, body=body, headers=headers)
     resp = conn.getresponse()
     resp.read()
 
@@ -364,9 +364,16 @@ def _upload_tarball_by_url_v2(staging_path, image_id, glance_endpoint,
         raise RetryableError(error)
 
     try:
-        _update_image_meta_v2(conn, image_id, extra_headers, properties)
+        mgt_url = "%(glance_endpoint)s/v2/images/%(image_id)s" % {
+            'glance_endpoint': glance_endpoint,
+            'image_id': image_id}
+        mgt_parts = urlparse(mgt_url)
+        mgt_path = mgt_parts[2]
 
-        validate_image_status_before_upload_v2(conn, url, extra_headers)
+        _update_image_meta_v2(conn, extra_headers, properties, mgt_path)
+
+        validate_image_status_before_upload_v2(conn, url, extra_headers,
+                                               mgt_path)
 
         try:
             conn.connect()
@@ -430,7 +437,6 @@ def check_resp_status_and_retry(resp, image_id, url):
                        httplib.UNAUTHORIZED,                     # 401
                        httplib.PAYMENT_REQUIRED,                 # 402
                        httplib.FORBIDDEN,                        # 403
-                       httplib.NOT_FOUND,                        # 404
                        httplib.METHOD_NOT_ALLOWED,               # 405
                        httplib.NOT_ACCEPTABLE,                   # 406
                        httplib.PROXY_AUTHENTICATION_REQUIRED,    # 407
@@ -454,6 +460,10 @@ def check_resp_status_and_retry(resp, image_id, url):
         raise PluginError("Got Permanent Error response [%i] while "
                           "uploading image [%s] to glance [%s]"
                           % (resp.status, image_id, url))
+    # Nova service would process the exception
+    elif resp.status == httplib.NOT_FOUND:                        # 404
+        exc = XenAPI.Failure('ImageNotFound')
+        raise exc
     # NOTE(nikhil): Only a sub-set of the 500 errors are retryable. We
     # optimistically retry on 500 errors below.
     elif resp.status in (httplib.REQUEST_TIMEOUT,                # 408
@@ -534,7 +544,8 @@ def validate_image_status_before_upload_v1(conn, url, extra_headers):
                                       'image_status': image_status})
 
 
-def validate_image_status_before_upload_v2(conn, url, extra_headers):
+def validate_image_status_before_upload_v2(conn, url, extra_headers,
+                                           get_path):
     try:
         parts = urlparse(url)
         path = parts[2]
@@ -545,8 +556,7 @@ def validate_image_status_before_upload_v2(conn, url, extra_headers):
         # it is not 'active' and send back a 409. Hence, the data will be
         # unnecessarily buffered by Glance. This wastes time and bandwidth.
         # LP bug #1202785
-
-        conn.request('GET', '/v2/images/%s' % image_id, headers=extra_headers)
+        conn.request('GET', get_path, headers=extra_headers)
         get_resp = conn.getresponse()
     except Exception, error:  # noqa
         logging.exception('Failed to GET the image %(image_id)s while '
