@@ -33,6 +33,10 @@ import json
 from os_xenapi.tests.plugins import plugin_test
 
 
+class FakeXenAPIException(Exception):
+    pass
+
+
 class Fake_HTTP_Request_Error(Exception):
     pass
 
@@ -326,19 +330,34 @@ class GlanceTestCase(plugin_test.PluginTestBase):
         fake_extra_headers = {}
         fake_properties = {}
         fake_endpoint = 'http://fake_netloc/fake_path'
+        fake_image_id = 'fake_image_id'
         expected_url = "%(glance_endpoint)s/v2/images/%(image_id)s/file" % {
             'glance_endpoint': fake_endpoint,
-            'image_id': 'fake_image_id'}
+            'image_id': fake_image_id}
+        expected_wsgi_path = '/fake_path/v2/images/%s' % fake_image_id
+        expect_url_parts = urlparse(expected_url)
+
+        expected_mgt_url = "%(glance_endpoint)s/v2/images/%(image_id)s" % {
+            'glance_endpoint': fake_endpoint,
+            'image_id': fake_image_id}
+        fake_mgt_parts = urlparse(expected_mgt_url)
+        fake_mgt_path = fake_mgt_parts[2]
 
         self.glance._upload_tarball_by_url_v2(
-            'fake_staging_path', 'fake_image_id', fake_endpoint,
+            'fake_staging_path', fake_image_id, fake_endpoint,
             fake_extra_headers, fake_properties)
 
-        self.assertTrue(mock_HTTPConn.called)
+        mock_HTTPConn.assert_called_with(expect_url_parts[0],
+                                         expect_url_parts[1])
         mock_validate_image.assert_called_with(fake_conn,
                                                expected_url,
-                                               fake_extra_headers)
-        self.assertTrue(mock_update_image_meta.called)
+                                               fake_extra_headers,
+                                               expected_wsgi_path)
+        mock_update_image_meta.assert_called_with(fake_conn,
+                                                  fake_extra_headers,
+                                                  fake_properties,
+                                                  fake_mgt_path)
+
         self.assertTrue(mock_create_tarball.called)
         self.assertTrue(
             mock_HTTPConn.return_value.getresponse.called)
@@ -362,24 +381,136 @@ class GlanceTestCase(plugin_test.PluginTestBase):
         fake_extra_headers = {}
         fake_properties = {}
         fake_endpoint = 'https://fake_netloc/fake_path'
+        fake_image_id = 'fake_image_id'
         expected_url = "%(glance_endpoint)s/v2/images/%(image_id)s/file" % {
             'glance_endpoint': fake_endpoint,
-            'image_id': 'fake_image_id'}
+            'image_id': fake_image_id}
+        expect_url_parts = urlparse(expected_url)
+        expected_wsgi_path = '/fake_path/v2/images/%s' % fake_image_id
+
+        expected_mgt_url = "%(glance_endpoint)s/v2/images/%(image_id)s" % {
+            'glance_endpoint': fake_endpoint,
+            'image_id': fake_image_id}
+        fake_mgt_parts = urlparse(expected_mgt_url)
+        fake_mgt_path = fake_mgt_parts[2]
 
         self.glance._upload_tarball_by_url_v2(
-            'fake_staging_path', 'fake_image_id', fake_endpoint,
+            'fake_staging_path', fake_image_id, fake_endpoint,
             fake_extra_headers, fake_properties)
 
-        self.assertTrue(mock_HTTPSConn.called)
-        self.assertTrue(mock_update_image_meta.called)
+        mock_update_image_meta.assert_called_with(fake_conn,
+                                                  fake_extra_headers,
+                                                  fake_properties,
+                                                  fake_mgt_path)
+        mock_HTTPSConn.assert_called_with(expect_url_parts[0],
+                                          expect_url_parts[1])
         mock_validate_image.assert_called_with(fake_conn,
                                                expected_url,
-                                               fake_extra_headers)
+                                               fake_extra_headers,
+                                               expected_wsgi_path)
         self.assertTrue(mock_create_tarball.called)
         self.assertTrue(
             mock_HTTPSConn.return_value.getresponse.called)
         self.assertFalse(mock_check_resp_status.called)
 
+    def test_upload_tarball_by_url_v2_with_api_endpoint(self):
+        fake_conn = mock.Mock()
+        mock_Conn = self.mock_patch_object(
+            self.glance, '_create_connection', fake_conn)
+        mock_validate_image = self.mock_patch_object(
+            self.glance, 'validate_image_status_before_upload_v2')
+        mock_create_tarball = self.mock_patch_object(
+            self.glance.utils, 'create_tarball')
+        mock_check_resp_status = self.mock_patch_object(
+            self.glance, 'check_resp_status_and_retry')
+        mock_update_image_meta = self.mock_patch_object(
+            self.glance, '_update_image_meta_v2')
+        self.glance._create_connection().getresponse = mock.Mock()
+        self.glance._create_connection().getresponse().status = \
+            httplib.NO_CONTENT
+        fake_extra_headers = {}
+        fake_properties = {}
+        fake_endpoint = 'https://fake_netloc:fake_port'
+        fake_image_id = 'fake_image_id'
+        expected_url = "%(glance_endpoint)s/v2/images/%(image_id)s/file" % {
+            'glance_endpoint': fake_endpoint,
+            'image_id': fake_image_id}
+        expect_url_parts = urlparse(expected_url)
+        expected_api_path = '/v2/images/%s' % fake_image_id
+
+        expected_mgt_url = "%(glance_endpoint)s/v2/images/%(image_id)s" % {
+            'glance_endpoint': fake_endpoint,
+            'image_id': fake_image_id}
+        fake_mgt_parts = urlparse(expected_mgt_url)
+        fake_mgt_path = fake_mgt_parts[2]
+
+        self.glance._upload_tarball_by_url_v2(
+            'fake_staging_path', fake_image_id, fake_endpoint,
+            fake_extra_headers, fake_properties)
+
+        mock_update_image_meta.assert_called_with(fake_conn,
+                                                  fake_extra_headers,
+                                                  fake_properties,
+                                                  fake_mgt_path)
+        mock_Conn.assert_called_with(expect_url_parts[0], expect_url_parts[1])
+        mock_validate_image.assert_called_with(fake_conn,
+                                               expected_url,
+                                               fake_extra_headers,
+                                               expected_api_path)
+        self.assertTrue(mock_create_tarball.called)
+        self.assertTrue(
+            mock_Conn.return_value.getresponse.called)
+        self.assertFalse(mock_check_resp_status.called)
+
+    def test_upload_tarball_by_url_v2_with_wsgi_endpoint(self):
+        fake_conn = mock.Mock()
+        mock_Conn = self.mock_patch_object(
+            self.glance, '_create_connection', fake_conn)
+        mock_validate_image = self.mock_patch_object(
+            self.glance, 'validate_image_status_before_upload_v2')
+        mock_create_tarball = self.mock_patch_object(
+            self.glance.utils, 'create_tarball')
+        mock_check_resp_status = self.mock_patch_object(
+            self.glance, 'check_resp_status_and_retry')
+        mock_update_image_meta = self.mock_patch_object(
+            self.glance, '_update_image_meta_v2')
+        self.glance._create_connection().getresponse = mock.Mock()
+        self.glance._create_connection().getresponse().status = \
+            httplib.NO_CONTENT
+        fake_extra_headers = {}
+        fake_properties = {}
+        fake_endpoint = 'https://fake_netloc/fake_path'
+        fake_image_id = 'fake_image_id'
+        expected_url = "%(glance_endpoint)s/v2/images/%(image_id)s/file" % {
+            'glance_endpoint': fake_endpoint,
+            'image_id': fake_image_id}
+        expect_url_parts = urlparse(expected_url)
+        expected_wsgi_path = '/fake_path/v2/images/%s' % fake_image_id
+
+        expected_mgt_url = "%(glance_endpoint)s/v2/images/%(image_id)s" % {
+            'glance_endpoint': fake_endpoint,
+            'image_id': fake_image_id}
+        fake_mgt_parts = urlparse(expected_mgt_url)
+        fake_mgt_path = fake_mgt_parts[2]
+
+        self.glance._upload_tarball_by_url_v2(
+            'fake_staging_path', fake_image_id, fake_endpoint,
+            fake_extra_headers, fake_properties)
+
+        mock_update_image_meta.assert_called_with(fake_conn,
+                                                  fake_extra_headers,
+                                                  fake_properties,
+                                                  fake_mgt_path)
+        mock_Conn.assert_called_with(expect_url_parts[0], expect_url_parts[1])
+        mock_validate_image.assert_called_with(fake_conn,
+                                               expected_url,
+                                               fake_extra_headers,
+                                               expected_wsgi_path)
+        self.assertTrue(mock_create_tarball.called)
+        self.assertTrue(
+            mock_Conn.return_value.getresponse.called)
+        self.assertFalse(mock_check_resp_status.called)
+
     def test_upload_tarball_by_url_https_failed_retry_v2(self):
         fake_conn = mock.Mock()
         mock_HTTPSConn = self.mock_patch_object(
@@ -398,25 +529,39 @@ class GlanceTestCase(plugin_test.PluginTestBase):
         fake_extra_headers = {}
         fake_properties = {}
         fake_endpoint = 'https://fake_netloc/fake_path'
+        fake_image_id = 'fake_image_id'
         expected_url = "%(glance_endpoint)s/v2/images/%(image_id)s/file" % {
             'glance_endpoint': fake_endpoint,
-            'image_id': 'fake_image_id'}
+            'image_id': fake_image_id}
+        expected_wsgi_path = '/fake_path/v2/images/%s' % fake_image_id
+
+        expected_mgt_url = "%(glance_endpoint)s/v2/images/%(image_id)s" % {
+            'glance_endpoint': fake_endpoint,
+            'image_id': fake_image_id}
+        expect_url_parts = urlparse(expected_url)
+        fake_mgt_parts = urlparse(expected_mgt_url)
+        fake_mgt_path = fake_mgt_parts[2]
 
         self.glance._upload_tarball_by_url_v2(
-            'fake_staging_path', 'fake_image_id', fake_endpoint,
+            'fake_staging_path', fake_image_id, fake_endpoint,
             fake_extra_headers, fake_properties)
 
-        self.assertTrue(mock_HTTPSConn.called)
-        self.assertTrue(mock_update_image_meta.called)
+        mock_update_image_meta.assert_called_with(fake_conn,
+                                                  fake_extra_headers,
+                                                  fake_properties,
+                                                  fake_mgt_path)
+        mock_HTTPSConn.assert_called_with(expect_url_parts[0],
+                                          expect_url_parts[1])
         mock_validate_image.assert_called_with(fake_conn,
                                                expected_url,
-                                               fake_extra_headers)
+                                               fake_extra_headers,
+                                               expected_wsgi_path)
         self.assertTrue(mock_create_tarball.called)
         self.assertTrue(
             mock_HTTPSConn.return_value.getresponse.called)
         self.assertTrue(mock_check_resp_status.called)
 
-    def test_update_image_meta_ok_v2(self):
+    def test_update_image_meta_ok_v2_using_api_service(self):
         fake_conn = mock.Mock()
         fake_extra_headers = {'fake_type': 'fake_content'}
         fake_properties = {'fake_path': True}
@@ -434,15 +579,45 @@ class GlanceTestCase(plugin_test.PluginTestBase):
         fake_headers.update(**fake_extra_headers)
         fake_conn.getresponse.return_value = mock.Mock()
         fake_conn.getresponse().status = httplib.OK
+        expected_api_path = '/v2/images/%s' % 'fake_image_id'
 
-        self.glance._update_image_meta_v2(fake_conn, 'fake_image_id',
-                                          fake_extra_headers, fake_properties)
+        self.glance._update_image_meta_v2(fake_conn, fake_extra_headers,
+                                          fake_properties, expected_api_path)
         fake_conn.request.assert_called_with('PATCH',
                                              '/v2/images/%s' % 'fake_image_id',
                                              body=fake_body_json,
                                              headers=fake_headers)
         fake_conn.getresponse.assert_called()
 
+    def test_update_image_meta_ok_v2_using_uwsgi_service(self):
+        fake_conn = mock.Mock()
+        fake_extra_headers = {'fake_type': 'fake_content'}
+        fake_properties = {'fake_path': True}
+        new_fake_properties = {'path': '/fake-path',
+                               'value': "True",
+                               'op': 'add'}
+        fake_body = [
+            {"path": "/container_format", "value": "ovf", "op": "add"},
+            {"path": "/disk_format", "value": "vhd", "op": "add"},
+            {"path": "/visibility", "value": "private", "op": "add"}]
+        fake_body.append(new_fake_properties)
+        fake_body_json = json.dumps(fake_body)
+        fake_headers = {
+            'Content-Type': 'application/openstack-images-v2.1-json-patch'}
+        fake_headers.update(**fake_extra_headers)
+        fake_conn.getresponse.return_value = mock.Mock()
+        fake_conn.getresponse().status = httplib.OK
+        expected_wsgi_path = '/fake_path/v2/images/%s' % 'fake_image_id'
+
+        self.glance._update_image_meta_v2(fake_conn, fake_extra_headers,
+                                          fake_properties, expected_wsgi_path)
+        fake_conn.request.assert_called_with('PATCH',
+                                             '/fake_path/v2/images/%s' %
+                                             'fake_image_id',
+                                             body=fake_body_json,
+                                             headers=fake_headers)
+        fake_conn.getresponse.assert_called()
+
     def test_check_resp_status_and_retry_plugin_error(self):
         mock_resp_badrequest = mock.Mock()
         mock_resp_badrequest.status = httplib.BAD_REQUEST
@@ -465,6 +640,17 @@ class GlanceTestCase(plugin_test.PluginTestBase):
             'fake_image_id',
             'fake_url')
 
+    def test_check_resp_status_and_retry_image_not_found(self):
+        mock_resp_badgateway = mock.Mock()
+        mock_resp_badgateway.status = httplib.NOT_FOUND
+        self.glance.XenAPI.Failure = FakeXenAPIException
+        self.assertRaises(
+            self.glance.XenAPI.Failure,
+            self.glance.check_resp_status_and_retry,
+            mock_resp_badgateway,
+            'fake_image_id',
+            'fake_url')
+
     def test_check_resp_status_and_retry_unknown_status(self):
         fake_unknown_http_status = -1
         mock_resp_other = mock.Mock()
@@ -564,7 +750,30 @@ class GlanceTestCase(plugin_test.PluginTestBase):
             mock_head_resp, fake_image_id, fake_url)
         mock_conn.request.assert_called_once()
 
-    def test_validate_image_status_before_upload_ok_v2(self):
+    def test_validate_image_status_before_upload_ok_v2_using_api_service(self):
+        mock_conn = mock.Mock()
+        fake_url = 'http://fake_host:fake_port/fake_path/fake_image_id'
+        mock_check_resp_status_and_retry = self.mock_patch_object(
+            self.glance, 'check_resp_status_and_retry')
+        mock_head_resp = mock.Mock()
+        mock_head_resp.status = httplib.OK
+        mock_head_resp.read.return_value = '{"status": "queued"}'
+        mock_conn.getresponse.return_value = mock_head_resp
+        fake_extra_headers = mock.Mock()
+        expected_api_path = '/v2/images/%s' % 'fake_image_id'
+
+        self.glance.validate_image_status_before_upload_v2(
+            mock_conn, fake_url, fake_extra_headers, expected_api_path)
+
+        self.assertTrue(mock_conn.getresponse.called)
+        self.assertEqual(
+            mock_head_resp.read.call_count, 2)
+        self.assertFalse(mock_check_resp_status_and_retry.called)
+        mock_conn.request.assert_called_with('GET',
+                                             '/v2/images/fake_image_id',
+                                             headers=fake_extra_headers)
+
+    def test_validate_image_status_before_upload_ok_v2_using_uwsgi(self):
         mock_conn = mock.Mock()
         fake_url = 'http://fake_host/fake_path/fake_image_id'
         mock_check_resp_status_and_retry = self.mock_patch_object(
@@ -574,14 +783,19 @@ class GlanceTestCase(plugin_test.PluginTestBase):
         mock_head_resp.read.return_value = '{"status": "queued"}'
         mock_conn.getresponse.return_value = mock_head_resp
 
+        fake_extra_headers = mock.Mock()
+        fake_patch_path = 'fake_patch_path'
+
         self.glance.validate_image_status_before_upload_v2(
-            mock_conn, fake_url, extra_headers=mock.Mock())
+            mock_conn, fake_url, fake_extra_headers, fake_patch_path)
 
         self.assertTrue(mock_conn.getresponse.called)
         self.assertEqual(
             mock_head_resp.read.call_count, 2)
         self.assertFalse(mock_check_resp_status_and_retry.called)
-        mock_conn.request.assert_called_once()
+        mock_conn.request.assert_called_with('GET',
+                                             'fake_patch_path',
+                                             headers=fake_extra_headers)
 
     def test_validate_image_status_before_upload_get_image_failed_v2(self):
         mock_conn = mock.Mock()
@@ -590,10 +804,11 @@ class GlanceTestCase(plugin_test.PluginTestBase):
         mock_head_resp = mock.Mock()
         mock_head_resp.status = httplib.OK
         mock_conn.getresponse.return_value = mock_head_resp
+        expected_wsgi_path = '/fake_path/v2/images/%s' % 'fake_image_id'
 
         self.assertRaises(self.glance.RetryableError,
                           self.glance.validate_image_status_before_upload_v2,
-                          mock_conn, fake_url, extra_headers=mock.Mock())
+                          mock_conn, fake_url, mock.Mock(), expected_wsgi_path)
         mock_conn.request.assert_called_once()
         mock_head_resp.read.assert_not_called()
         mock_conn.getresponse.assert_not_called()
@@ -605,9 +820,10 @@ class GlanceTestCase(plugin_test.PluginTestBase):
         mock_head_resp = mock.Mock()
         mock_head_resp.status = httplib.BAD_REQUEST
         mock_conn.getresponse.return_value = mock_head_resp
+        expected_wsgi_path = '/fake_path/v2/images/%s' % 'fake_image_id'
 
         self.glance.validate_image_status_before_upload_v2(
-            mock_conn, fake_url, extra_headers=mock.Mock())
+            mock_conn, fake_url, mock.Mock(), expected_wsgi_path)
         mock_conn.request.assert_called_once()
         mock_conn.getresponse.assert_called_once()
         mock_head_resp.read.assert_called_once()
@@ -620,10 +836,11 @@ class GlanceTestCase(plugin_test.PluginTestBase):
         mock_head_resp.status = httplib.OK
         mock_head_resp.read.return_value = '{"status": "not-queued"}'
         mock_conn.getresponse.return_value = mock_head_resp
+        expected_wsgi_path = '/fake_path/v2/images/%s' % 'fake_image_id'
 
         self.assertRaises(self.glance.PluginError,
                           self.glance.validate_image_status_before_upload_v2,
-                          mock_conn, fake_url, extra_headers=mock.Mock())
+                          mock_conn, fake_url, mock.Mock(), expected_wsgi_path)
         mock_conn.request.assert_called_once()
         mock_head_resp.read.assert_called_once()
 
